def get_depTable():
    
    import os
    import pandas as pd
    
    # Get current working directory
    currDir = os.getcwd()
    
    # Change working directory to location of deposition lookup tables
    depTableDir = os.path.dirname(__file__) + '/Deposition Lookup Tables'
    os.chdir(depTableDir)
    
    # Get lookup tables
    headLUT = pd.read_csv('HeadDeposition.csv')
    lungLUT = pd.read_csv('LungDeposition.csv')
    
    # Change back to original working directory
    os.chdir(currDir)
    
    return headLUT, lungLUT

def extrap1d(interpolator):
    
    import scipy
    from scipy import array
    
    xs = interpolator.x
    ys = interpolator.y

    def pointwise(x):
        if x < xs[0]:
            return ys[0]+(x-xs[0])*(ys[1]-ys[0])/(xs[1]-xs[0])
        elif x > xs[-1]:
            return ys[-1]+(x-xs[-1])*(ys[-1]-ys[-2])/(xs[-1]-xs[-2])
        else:
            return interpolator(x)

    def ufunclike(xs):
        return array(list(map(pointwise, array(xs))))

    return ufunclike

def InterpOverGrid(x, y, v, xq, yq):
    
    import scipy.interpolate as interp
    
    F = interp.interp2d(x, y, v, kind='linear')
    vq_Full = F(xq, yq)
    vq = vq_Full[0]
    
    return vq

def DepositionCalc(diameters, minuteVentilation_ndz, inhalationFactors):
    
    import numpy as np
    import warnings
    warnings.filterwarnings("ignore")
    
    headLUT, lungLUT = get_depTable()
    
    minVentNDZ_vec = np.ones(diameters.shape)*minuteVentilation_ndz
    
    # Deposition fractions for the head
    DE_H = InterpOverGrid(np.log10(headLUT['ParticleDiameter_In']), headLUT['VentilationRate'], headLUT['DE_URT'], np.log10(diameters), minVentNDZ_vec)
    DF_H = DE_H
    
    # Interpolate output diameters from lookup table
    diametersOut_log10 = InterpOverGrid(np.log10(headLUT['ParticleDiameter_In']), headLUT['VentilationRate'], np.log10(headLUT['ParticleDiameter_Out']), np.log10(diameters), minVentNDZ_vec)
    
    # Deposition fractions for TB 
    DE_TB = InterpOverGrid(np.log10(lungLUT['ParticleDiameter_Out']), lungLUT['VentilationRate'], lungLUT['DE_TB'], diametersOut_log10, minVentNDZ_vec)
    DF_TB = np.multiply(DE_TB, (1-DE_H))
    
    # Deposition fractions for P
    DE_P = InterpOverGrid(np.log10(lungLUT['ParticleDiameter_Out']), lungLUT['VentilationRate'], lungLUT['DE_P'], diametersOut_log10, minVentNDZ_vec)
    DF_P = np.multiply(DE_P, (1-DE_H))
    
    return DF_H, DF_TB, DF_P
