def GetDiametersPreEvap(m, n, d0):
    
    import numpy as np
    import math
    
    out = np.zeros([n])
    
    for i in range(n):
        e = i/m
        out[i] = d0[i]*np.power(10, math.e)
    
    return out

def ComputeDiametersCDF(diameters, cmd, gsd):
    
    import numpy as np
    from math import erf
    
    X = np.divide(np.log(diameters/cmd), np.log(gsd)*np.sqrt(2))
    cdf = np.zeros([len(X)])
    for i in range(len(X)):
        cdf[i] = 0.5*(1+erf(X[i]))
    
    return cdf

def ComputeDiametersParticleConc(diametersCDF, concNum):
    
    pConc = np.zeros([len(diametersCDF)])
    
    for i in range(len(diametersCDF)):
        if i==0:
            pConc[i] = concNum*diametersCDF[i]
        else:
            pConc[i] = concNum*(diametersCDF[i]-diametersCDF[i-1])
            
    return pConc

def SizeDistributionCalc_OLD(m, n, params):
    
    diametersPreEvap = GetDiametersPreEvap(m, n, params['d0'])
    
    diametersCDF_bronch = ComputeDiametersCDF(diametersPreEvap, params['cmd_bronch'], params['gsd_bronch'])
    diametersCDF_laryn = ComputeDiametersCDF(diametersPreEvap, params['cmd_laryn'], params['gsd_laryn'])
    diametersCDF_oral = ComputeDiametersCDF(diametersPreEvap, params['cmd_oral'], params['gsd_oral'])
    
    particleConc_bronch = ComputeDiametersParticleConc(diametersCDF_bronch, params['concNum_bronch'])
    particleConc_laryn = ComputeDiametersParticleConc(diametersCDF_laryn, params['concNum_laryn'])
    particleConc_oral = ComputeDiametersParticleConc(diametersCDF_oral, params['concNum_oral'])
    
    out = particleConc_bronch + particleConc_laryn + particleConc_oral
    
    return out

def SizeDistributionCalc(diameters, params, get_allConc=False):
    
    import numpy as np, scipy as sp
    import scipy.special
    import math
    
    MD_B, GSD_B, C_B = params['cmd_bronch'], params['gsd_bronch'], params['concNum_bronch']
    MD_L, GSD_L, C_L = params['cmd_laryn'], params['gsd_laryn'], params['concNum_laryn']
    MD_O, GSD_O, C_O = params['cmd_oral'], params['gsd_oral'], params['concNum_oral']
    
    cdf1, cdf2, cdf3 = np.zeros(len(diameters)), np.zeros(len(diameters)), np.zeros(len(diameters))
    conc1, conc2, conc3 = np.zeros(len(diameters)), np.zeros(len(diameters)), np.zeros(len(diameters))
    prob, total_conc = np.zeros(len(diameters)), np.zeros(len(diameters))
    
    if params['emissionMech']=='breathing':
        for i, d in enumerate(diameters):
            prob[i] = np.multiply(np.divide(1,np.multiply(d, math.log(GSD_B)*np.sqrt(2*np.pi))),np.power(math.e, (-np.power(np.math.log(d/MD_B),2))/(2*np.power(math.log(GSD_B),2))))
            cdf1[i] = (1/2)*(1+sp.special.erf(np.divide(np.log(np.divide(d,MD_B)),np.log(GSD_B)*np.sqrt(2))))
            if i==0:
                conc1[i] = np.multiply(C_B, cdf1[i])
            else:
                conc1[i] = np.multiply(C_B,(cdf1[i]-cdf1[i-1]))
            total_conc[i] = conc1[i]
    
    elif params['emissionMech']=='coughing':
        for i, d in enumerate(diameters):
            prob[i] = np.multiply(np.divide(1,np.multiply(d, math.log(GSD_B)*np.sqrt(2*np.pi))),np.power(math.e, (-np.power(np.math.log(d/MD_B),2))/(2*np.power(math.log(GSD_B),2))))
            cdf1[i] = (1/2)*(1+sp.special.erf(np.divide(np.log(np.divide(d,MD_B)),np.log(GSD_B)*np.sqrt(2))))
            cdf2[i] = (1/2)*(1+sp.special.erf(np.divide(np.log(np.divide(d,MD_L)),np.log(GSD_L)*np.sqrt(2))))
            cdf3[i] = (1/2)*(1+sp.special.erf(np.divide(np.log(np.divide(d,MD_O)),np.log(GSD_O)*np.sqrt(2))))
            if i==0:
                conc1[i] = np.multiply(C_B, cdf1[i])
                conc2[i] = np.multiply(C_L, cdf2[i])
                conc3[i] = np.multiply(C_O, cdf3[i])
            else:
                conc1[i] = np.multiply(C_B,(cdf1[i]-cdf1[i-1]))
                conc2[i] = np.multiply(C_L,(cdf2[i]-cdf2[i-1]))
                conc3[i] = np.multiply(C_O,(cdf3[i]-cdf3[i-1]))
            total_conc[i] = conc1[i] + conc2[i] + conc3[i]
        
    # cD = {'Diameters': diameters, 'CDF1': cdf1, 'Conc1': conc1, 'CDF2': cdf2, 'Conc2': conc2, 'CDF3': cdf3, 'Conc3': conc3, 'Total Conc': total_conc}
    # pd.DataFrame(cD).to_excel('sizeDist_test.xlsx')
    if get_allConc:
        if params['emissionMech']=='breathing':
            out = {'conc1': conc1, 'total_conc': total_conc}
        elif params['emissionMech']=='coughing':
            out = {'conc1': conc1, 'conc2': conc2, 'conc3': conc3, 'total_conc': total_conc}
    else:
        out = total_conc
        
    return out
